rng shuffle

DEBUG = 0;%~flag_capture_oscilloscope;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Includes
addpath(genpath('./utils'));
addpath(genpath('./utils/bin2hex'));
addpath(genpath('./utils/AES'));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Variable Initilization

% The first plaintext that will be given to NUCLEO for encryption - Verification
% New ones need to be generated before another run
if (i == 0)
    plaintext_hex = {'5b' '7e' '15' '16' '28' 'ae' 'd2' 'a6' 'ab' 'f7' '15' '88' '09' 'cf' '4f' '3c'};
    plaintext = hex2dec(plaintext_hex);
else
    % New plaintext generation
    random_plaintext_bin = num2str(randi(2,1,128)-1);
    random_plaintext_bin(strfind(random_plaintext_bin,' ')) = [];
    byte_split = reshape(random_plaintext_bin,16,8);
    for i = 1:16
    plaintext(i) = bin2dec(byte_split(i,:));
    end
    plaintext = plaintext.';
end

% The 'secret' key - used for verification only
% Make sure the same is used in the device
key_hex = {'fc' '7e' '15' '16' '2d' 'ae' 'd2' 'a6' 'ab' 'f7' '15' '88' '09' 'cf' '4f' '3c'};
key = hex2dec(key_hex);

% Create datapoints matrix to save trace points for each trace in dataset
datapoints = double(zeros(numofencryptions, lecroy_num_of_points - 18));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Initialize Devices Communication
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialize serial port
myComPort = serialport(COM_PORT_NUMBER,BAUD_RATE);

% Initialize oscilloscope
if flag_capture_oscilloscope == 1;
    init_SIGLENT_scope;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Create result cell array for the following columns

% Below you will define:
% col 1 : Key used for MATLAB (and attack platform) computation of the AES in hex format
% col 2 : random plaintext generated by MATLAB  in hex format
% col 3 : Verification ciphertext (only for the first computation) in hex format
% col 4 : Ciphertext as computed by the STM32 in hex format

result = {};

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Execution
% For all traces in dataset
for enc_num = 1:numofencryptions

    plaintexts_SCA(enc_num,:) = plaintext.'; %for SCA algorithm

    % Pause for 10 ns
    pause(0.001);

    write_plaintext;

    % Data acquisition from oscilloscope
    % Capture data from SIGLENT and save them to datapoints
    if flag_capture_oscilloscope == 1
        acquire_SIGLENT_scope_data_CH1;
        datapoints(enc_num,:)  = C1_num_fixed2;
    end


    %Save data in result matrix - hexadecimal form
    p = dec2hex(plaintext.');

    c = dec2hex(ciphertext.');

    result{enc_num,1} = cell2mat(key_hex);
    result{enc_num,2} = strcat(p(1,:),p(2,:),p(3,:),p(4,:),p(5,:),p(6,:),p(7,:),p(8,:),p(9,:),p(10,:),p(11,:),p(12,:),p(13,:),p(14,:),p(15,:),p(16,:));

    result{enc_num,4} = strcat(c(1,:),c(2,:),c(3,:),c(4,:),c(5,:),c(6,:),c(7,:),c(8,:),c(9,:),c(10,:),c(11,:),c(12,:),c(13,:),c(14,:),c(15,:),c(16,:));

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    %Verify AES from Matlab - only for the first ciphertext

%     if ((enc_num == 1 && i == 0)||DEBUG == 1)
% 
%         % Initialization - AES scipt are defined in ./utils/AES folder
%         [s_box, inv_s_box, w, poly_mat, inv_poly_mat] = aes_init(key_hex);
% 
%         % Encrypt the plaintext to ciphertext through Matlab
%         ciphertext = cipher (plaintext, w, s_box, poly_mat,1).';
%         c = dec2hex(ciphertext);
%         verification_ciphertext = strcat(c(1,:),c(2,:),c(3,:),c(4,:),c(5,:),c(6,:),c(7,:),c(8,:),c(9,:),c(10,:),c(11,:),c(12,:),c(13,:),c(14,:),c(15,:),c(16,:));
%         result{enc_num,3} = verification_ciphertext;
%         verification_ciphertext = ''; %not used beyond the first trace
% 
%         %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%         % Error if MATLAB ciphertext does not correspond to NUCLEO
% 
%         if(strcmp(result{enc_num,3},result{enc_num,4}) == 0)
%             error ('Error : Encryption verification failed')
%         end
% 
%         % If not the first encryption check whether save dataset or
%         % save trace info and generate new plaintexts
%     else
        if (enc_num > 1)
        % Save dataset every numofencryptions traces
        if mod(enc_num,numofencryptions) == 0

            if flag_capture_oscilloscope == 1
                % Name of file format
                file_name = strcat('attack_data_',num2str(file_number,'%d'),'.mat');

                % Save dataset : datapoints, and result
                save([pwd strcat('/attack_data/',file_name)], 'datapoints','plaintexts_SCA','result');

                % Clear datapoints and result and for dataset
                clear datapoints;
            end
            clear plaintexts_SCA;
            clear result;
        end
    end


    % New plaintext generation
    random_plaintext_bin = num2str(randi(2,1,128)-1);
    random_plaintext_bin(strfind(random_plaintext_bin,' ')) = [];
    byte_split = reshape(random_plaintext_bin,16,8);
    for i = 1:16
        plaintext(i) = bin2dec(byte_split(i,:));
    end
    plaintext = plaintext.';
    %%%
end


if (flag_capture_oscilloscope == 1)
    fclose(vu);
    delete(vu);
    clear vu;
end




